[btr] Only print debug message if we're really increasing the wrap buffer.
[paraslash.git] / ipc.c
diff --git a/ipc.c b/ipc.c
index 49d2d96515dbf49bc5a9f7ace0786d5f68acc936..c64cc89c6d8803416a9686890b09592c97a9272e 100644 (file)
--- a/ipc.c
+++ b/ipc.c
@@ -1,22 +1,10 @@
 /*
- * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2006-2009 Andre Noll <maan@systemlinux.org>
  *
- *     This program is free software; you can redistribute it and/or modify
- *     it under the terms of the GNU General Public License as published by
- *     the Free Software Foundation; either version 2 of the License, or
- *     (at your option) any later version.
- *
- *     This program is distributed in the hope that it will be useful,
- *     but WITHOUT ANY WARRANTY; without even the implied warranty of
- *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *     GNU General Public License for more details.
- *
- *     You should have received a copy of the GNU General Public License
- *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Licensed under the GPL v2. For licencing details see COPYING.
  */
 
-/** \file ipc.c interprocess communication and shared memory helpers */
+/** \file ipc.c Inter-process communication and shared memory helpers. */
 
 #include "para.h"
 #include "error.h"
 #include <sys/shm.h>
 #include <sys/sem.h>
 
-/** abort if semget() failed that many times */
-#define MAX_SEMOP_RETRIES 500
-
 /**
- * define a new mutex
+ * Define a new mutex.
  *
* \return the identifier for the new mutex on success, -E_SEM_GET
  * on errors.
  *
- * \sa semget(2)
+ * \sa semget(2).
  */
 int mutex_new(void)
 {
        int ret = semget(IPC_PRIVATE, 1, IPC_CREAT | 0666);
-       return ret < 0?  -E_SEM_GET : ret;
+       return ret < 0? -ERRNO_TO_PARA_ERROR(errno) : ret;
 }
 
 /**
- * destroy a mutex
+ * Destroy a mutex.
  *
- * \param id the identifier of the mutex to be destroyed
+ * \param id The identifier of the mutex to be destroyed.
  *
- * \returns Positive on success, -E_SEM_REMOVE on errors.
+ * \return Standard.
  *
  * \sa semctl(2)
  */
 int mutex_destroy(int id)
 {
        int ret = semctl(id, 0, IPC_RMID);
-       return ret < 0? -E_SEM_REMOVE : 1;
+       return ret < 0? -ERRNO_TO_PARA_ERROR(errno) : 1;
 }
 
 static void para_semop(int id, struct sembuf *sops, int num)
 {
-       int i;
-
-       for (i = 0; i < MAX_SEMOP_RETRIES; i++)
+       do {
                if (semop(id, sops, num) >= 0)
                        return;
-       PARA_EMERG_LOG("semop failed %d times, aborting\n", MAX_SEMOP_RETRIES);
+       } while (errno == EINTR);
+       if (errno == EIDRM) {
+               PARA_CRIT_LOG("semaphore set %d was removed\n", id);
+               return;
+       }
+       PARA_EMERG_LOG("fatal semop error %s: pid %d\n", strerror(errno),
+               (int)getpid());
        exit(EXIT_FAILURE);
 }
 
 /**
- * lock the given mutex
+ * Lock the given mutex.
+ *
+ * \param id The identifier of the shared memory area to lock.
  *
  * This function either succeeds or aborts.
  *
- * \sa semop(2), struct misc_meta_data
+ * \sa semop(2), struct misc_meta_data.
  */
 void mutex_lock(int id)
 {
@@ -93,13 +84,13 @@ void mutex_lock(int id)
 }
 
 /**
- * unlock a mutex
+ * Unlock a mutex.
  *
- * \param id the identifier of the mutex
+ * \param id The identifier of the mutex.
  *
  * This function either succeeds or aborts.
  *
- * \sa semop(2), struct misc_meta_data
+ * \sa semop(2), struct misc_meta_data.
  */
 void mutex_unlock(int id)
 {
@@ -114,59 +105,69 @@ void mutex_unlock(int id)
 }
 
 /**
* create a new shared memory area of given size
* 
* \sa shmget(2)
  */
 int shm_new(size_t size)
 {
        int ret = shmget(IPC_PRIVATE, size, IPC_CREAT | IPC_EXCL | 0600);
-       return ret < 0 ? -E_SHM_GET : ret;
+       return ret < 0 ? -ERRNO_TO_PARA_ERROR(errno) : ret;
 }
 
 /**
- * destroy the given shared memory area
- * \sa shmctl(2)
- **/
+ * Destroy the given shared memory area.
+ *
+ * \param id The shared memory identifier.
+ *
+ * \return The return value of the underlying shmctl() call on success,
+ * a negative error code on errors.
+ *
+ * \sa shmctl(2).
+ */
 int shm_destroy(int id)
 {
        struct shmid_ds shm_desc;
        int ret = shmctl(id, IPC_RMID, &shm_desc);
-       return ret < 0? -E_SHM_DESTROY : ret;
+       return ret < 0? -ERRNO_TO_PARA_ERROR(errno) : ret;
 }
 
 /**
- * attach a shared memory segment
+ * Attach a shared memory segment.
  *
- * \param id the identifier of the shared memory segment to attach
- * \param mode either ATTACH_RO (read only) or ATTACH_RW (read/write)
- * \param result points to the attached arer which to the
+ * \param id The identifier of the shared memory segment to attach.
+ * \param mode Either ATTACH_RO (read only) or ATTACH_RW (read/write).
+ * \param result Points to the attached area just attached on success.
  *
- * \returns positive on success, -E_SHM_ATTACH on errrors.
+ * \return Standard.
  *
- * \sa shmat(2)
+ * \sa shmat(2).
  */
 int shm_attach(int id, enum shm_attach_mode mode, void **result)
 {
-       if (mode == ATTACH_RW) {
+       if (mode == ATTACH_RW)
                *result = shmat(id, NULL, 0);
-               return *result? 1 : -E_SHM_ATTACH;
-       }
-       *result = shmat(id, NULL, SHM_RDONLY);
-       return *result? 1 : -E_SHM_ATTACH;
+       else
+               *result = shmat(id, NULL, SHM_RDONLY);
+       return *result == (void *) -1? -ERRNO_TO_PARA_ERROR(errno) : 1;
 }
 
 /**
- * detach a shared memory segment
+ * Detach a shared memory segment.
  *
- * \param addr the address of the attached segment
+ * \param addr The address of the attached segment.
  *
- * \returns positive on success, -E_SHM_DETACH on errors.
+ * \return Standard.
  *
- * \sa shmdt(2)
+ * \sa shmdt(2).
  */
 int shm_detach(void *addr)
 {
        int ret = shmdt(addr);
-       return ret < 0? -E_SHM_DETACH : 1;
+       return ret < 0? -ERRNO_TO_PARA_ERROR(errno) : 1;
 }
@@ -1,22 +1,10 @@
  *
- * \return the identifier for the new mutex on success, -E_SEM_GET
+ * \return The identifier for the new mutex on success, a negative error code
  * on errors.
  *
@@ -114,59 +105,69 @@ void mutex_unlock(int id)
 
 /**
- * create a new shared memory area of given size
- * 
- * \sa shmget(2)
+ * Create a new shared memory area of given size.
+ *
+ * \param size The size of the shared memory area to create.
+ *
+ * \return The id of the shared memory array on success, a negative error
+ * code on errors.
+ *
+ * \sa shmget(2).
  */
 int shm_new(size_t size)